;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;; See the file LICENCE for licence information.

;; TODO
;;  What to do with LC_CTYPE, LC_COLLATE
;;  Test on windows.
;;  Parsers?

(in-package :cl-l10n )

;; Variables 
(defvar *locale-path* 
  (let ((path *load-pathname*))
    (make-pathname :host (pathname-host path)
                   :device (pathname-device path)
                   :directory 
                   (append (pathname-directory path)
                           '("locales"))
                   :defaults #P"")))
                  

(defvar *locale* nil)

(defvar *locales* (make-hash-table :test #'equal))

;; Conditions
(defun locale-report (obj stream)
  (format stream "~A" (mesg obj)))

(define-condition locale-error ()
  ((mesg :accessor mesg :initarg :mesg :initform "Unknown."))
  (:report locale-report))

(defun locale-error (string &rest args)
  (error 'locale-error :mesg (apply #'format nil string args)))


;; Classes
(defclass locale ()
  ((locale-name :accessor locale-name :initarg :name 
                :initform (required-arg :name))
   (title :accessor title :initarg :title :initform nil)
   (source :accessor source :initarg :source :initform nil)
   (language :accessor language :initarg :language :initform nil)
   (territory :accessor territory :initarg :territory :initform nil)
   (revision :accessor revision :initarg :revision :initform nil)
   (date :accessor date :initarg :date :initform nil)
   (categories :accessor categories :initarg :categories
               :initform (make-hash-table :test #'equal))))

(defmethod print-object ((obj locale) stream)
  (print-unreadable-object (obj stream :type t :identity t)
    (princ (locale-name obj) stream)))

(defclass category ()
  ((category-name :accessor category-name :initform (required-arg :name) 
                  :initarg :name)
   (vals :accessor vals :initform (make-hash-table :test #'equal)
         :initarg :vals)))

(defmethod print-object ((obj category) stream)
  (print-unreadable-object (obj stream :type t :identity t)
    (princ (category-name obj) stream)))


;; Macros
(defmacro get-locale (name)
  `(gethash ,name *locales*))

(defmacro get-category (name locale)
  `(gethash ,name (categories ,locale)))

(defmacro get-cat-val (value cat)
  `(gethash ,value (vals ,cat)))


(defun locale-value (locale cat key)
  (awhen (get-category cat locale)
    (get-cat-val key it)))

(defun getenv (word)
  #+sbcl (sb-ext:posix-getenv word)
  #+lispworks (hcl:getenv word)
  #+cmu (cdr (assoc (intern word :keyword) ext:*environment-list*))
  #+clisp (ext:getenv word))


;; Getters
(defmacro defgetter (key cat &key wrap)
  (let ((name (symb "LOCALE-" (substitute #\- #\_ (string-upcase key)))))
    `(progn 
      (defun ,name (&optional (locale *locale*))
        (let ((locale (locale-des->locale locale)))
          (when locale
            (awhen (get-category ,cat locale)
              ,(if wrap
                   `(funcall ,wrap (get-cat-val ,key it))
                   `(get-cat-val ,key it))))))
      (export ',name))))

(defun parse-car-or-val (x)
  (values (parse-integer (if (consp x) (car x) x))))

(defgetter "int_curr_symbol" "LC_MONETARY")
(defgetter "currency_symbol" "LC_MONETARY")
(defgetter "mon_decimal_point" "LC_MONETARY")
(defgetter "mon_thousands_sep" "LC_MONETARY")
(defgetter "mon_grouping" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "positive_sign" "LC_MONETARY")
(defgetter "negative_sign" "LC_MONETARY")
(defgetter "int_frac_digits" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "frac_digits" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "p_cs_precedes" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "p_sep_by_space" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "n_cs_precedes" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "n_sep_by_space" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "p_sign_posn" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "n_sign_posn" "LC_MONETARY" :wrap 'parse-car-or-val)
(defgetter "decimal_point" "LC_NUMERIC")
(defgetter "thousands_sep" "LC_NUMERIC")
(defgetter "grouping" "LC_NUMERIC" :wrap 'parse-car-or-val)
(defgetter "abday" "LC_TIME")
(defgetter "day" "LC_TIME")
(defgetter "abmon" "LC_TIME")
(defgetter "mon" "LC_TIME")
(defgetter "d_t_fmt" "LC_TIME")
(defgetter "d_fmt" "LC_TIME")
(defgetter "t_fmt" "LC_TIME")
(defgetter "am_pm" "LC_TIME")
(defgetter "t_fmt_ampm" "LC_TIME")
(defgetter "date_fmt" "LC_TIME")
(defgetter "yesexpr" "LC_MESSAGES")
(defgetter "noexpr" "LC_MESSAGES")
(defgetter "height" "LC_PAPER")
(defgetter "width" "LC_PAPER")
(defgetter "name_fmt" "LC_NAME")
(defgetter "name_gen" "LC_NAME")
(defgetter "name_mr" "LC_NAME")
(defgetter "name_mrs" "LC_NAME")
(defgetter "name_miss" "LC_NAME")
(defgetter "name_ms" "LC_NAME")
(defgetter "postal_fmt" "LC_ADDRESS")
(defgetter "tel_int_fmt" "LC_TELEPHONE")
(defgetter "measurement" "LC_MEASUREMENT")


;; EOF